<?php

namespace GopNext\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;

class PercentageAndStructureForBudgetController {
    protected $crudService;
    
    public function __construct($connection, $environment, $service, $util, $crudService) {
        $this->connection   = $connection;
        $this->environment  = $environment;
        $this->service      = $service;
        $this->util         = $util;
        $this->crudService	= $crudService;
    }
    
    public function structureItemChange(Request\DataSet $request, Response $response) {
      $data = $request->getDataSet()->getRows();
      
      try {
        $this->service->beginTransaction();
        foreach ($data as $row) {
          $params = [
            "VRPERCITEMGOP"   => $row["VRPERCITEMGOP"],
            "CDFILIAL"        => $row["CDFILIAL"],
            "NRSEQITEMORCA"   => $row["NRSEQITEMORCA"],
            "CDOPERADOR"      => $this->environment->getCdOperador(),
            "NRORG"           => $this->environment->getNrorg()
          ];
          $this->service->updateOperationalItem($params, $row["NRSEQ"]);
        }
        $this->service->commit();
      } catch (\Exception $ex) {
          $this->service->rollback();
          $response->setError(new Error($ex->getMessage(), 500));
      }
    }
    
    public function detailsOperationalItemsStructure(Request\Filter $request, Response $response) {
        $params = ['NRORG'];
        $conditions = $this->util->prepareConditions($params);
        $filter = $request->getFilterCriteria()->getConditions();
        
        /*
        foreach ($filter as &$item) {
            if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                $item["value"] = $item["value"]["NRPERIODOORCA"];
            }
        }
        */
        $data = $this->crudService->getFilteredDataSet('detailsOperationalItemsStructure', array_merge($conditions, $filter))->getRows();
		$response->addDataSet(new DataSet("detailsOperationalItemsStructure", $data));
    }
    
    public function budgetUnits(Request\Filter $request, Response $response) {
        try {
            $this->service->beginTransaction();
            $params = ['CDOPERADOR', 'NRORG'];
            $conditions = $this->util->prepareConditions($params);
            $filter = $request->getFilterCriteria()->getConditions();
            $this->scriptSequence();
            $this->checksRecord('P');
            
            /*foreach ($filter as &$item) {
                if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                    $item["value"] = $item["value"]["NRPERIODOORCA"];
                }
            }*/
            $data = $this->crudService->getFilteredDataSet('operationalStructureUnit', array_merge($conditions, $filter))->getRows();
    		$response->addDataSet(new DataSet("budgetUnits", $data));
    		$this->service->commit();
        } catch (\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function scriptSequence() {
        $arrayTable = ['PERCITFILGOP', 'PERCITDETALFILGOP', 'PERCITEMDETALGOP', 'PERCDETPRODFILGOP', 'PERCDETALPRODGOP', 'APROVAORCAGOP'];
        foreach($arrayTable as $table) {
            $tableSequence = $table;
            $this->service->executeScriptSequence($tableSequence);
        }
    }
    
    public function checksRecord($cdFilial) {
        $nrOrg = $this->environment->getNrorg();
        if($cdFilial === 'P')
        {
            $result = $this->service->selectItemsOperational();
        } else {
            $params = ["CDFILIAL" => $cdFilial];
            $result = $this->service->selectItemsOperationalUnit($params);
            
        }
        $arrayNrSeq = [];
        
        if(count($result) !== 11) {
            foreach($result as $nrSeq) {
                array_push($arrayNrSeq, $nrSeq['NRSEQITEMORCA']);
            }
            
            $arrayItems = [
                array("NRPERCITEMGOP" => 0, "NRSEQITEMORCA" => 0, "DSITEMORCAMENTO" => 'Faturamento', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 1, "NRSEQITEMORCA" => 1, "DSITEMORCAMENTO" => 'Imposto', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 2, "NRSEQITEMORCA" => 2, "DSITEMORCAMENTO" => 'Consumo', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 3, "NRSEQITEMORCA" => 3, "DSITEMORCAMENTO" => 'Mão de Obra', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 5, "NRSEQITEMORCA" => 5, "DSITEMORCAMENTO" => 'Despesas da Matriz', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 6, "NRSEQITEMORCA" => 6, "DSITEMORCAMENTO" => 'Despesas Diversas', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 7, "NRSEQITEMORCA" => 7, "DSITEMORCAMENTO" => 'Despesas Lanc. Entrada', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 8, "NRSEQITEMORCA" => 8, "DSITEMORCAMENTO" => 'Imposto a Recuperar', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 9, "NRSEQITEMORCA" => 9, "DSITEMORCAMENTO" => 'Impostos Sobre o Lucro', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 10, "NRSEQITEMORCA" => 10, "DSITEMORCAMENTO" => 'Investimentos', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg),
                array("NRPERCITEMGOP" => 11, "NRSEQITEMORCA" => 11, "DSITEMORCAMENTO" => 'Depreciação', "VRPERCITEMGOP" => 0.0, "CDFILIAL" => $cdFilial, "NRORG"=> $nrOrg)
            ];
            foreach($arrayItems as $item) {
                if(array_search($item['NRSEQITEMORCA'], $arrayNrSeq) === false) {
                    if ($cdFilial === 'P') {
                        unset($item["CDFILIAL"]);
                        $this->service->insertItemsOperational($item);
                    } else {
                        unset($item["NRPERCITEMGOP"]);
                        $this->service->insertItemsOperationalUnit($item);
                    }
                }
            }
        }
    }
    
    public function operationalItemStructure(Request\Filter $request, Response $response) {
        try {
            $this->service->beginTransaction();
            $params = ['NRORG'];
            $conditions = $this->util->prepareConditions($params);
            $filter = $request->getFilterCriteria()->getConditions();
            
            /*
            foreach ($filter as &$item) {
                if ($item["columnName"] === "NRPERIODOORCA" && is_array($item["value"])) {
                    $item["value"] = $item["value"]["NRPERIODOORCA"];
                }
            }
            */
            $this->checksRecord($filter[0]["value"]);
            $data = $this->crudService->getFilteredDataSet('operationalItemStructure', array_merge($conditions, $filter))->getRows();
    		$response->addDataSet(new DataSet("operationalItemStructure", $data));
    		$this->service->commit();
        } catch (\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function updateDetail(Request\DataSet $request, Response $response) {
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->beginTransaction();
            $operator = $this->environment->getCdOperador();
            $nrOrg = $this->environment->getNrorg();
            $rowItemOrca = $rows[1];
            $cdFilial = $rowItemOrca["CDFILIAL"];
            foreach($rows[0] as $row) {
                $origin = $row["DSORIGEM"];
                $newRegister = !isset($row['NRSEQITEMOP']);
                $cdProduto = $row["DSORIGEM"] === 'PRODUTO' ? $row["CDCODIGO"] : null;
                $cdImposto = $row["DSORIGEM"] === 'IMPOSTO' ? $row["CDCODIGO"] : null;
                $cdTipoOper = $row["DSORIGEM"] === 'TIPOOPERAC' ? $row["CDCODIGO"] : null;
                $idEntrSaidOp = isset($row["IDENTRSAIDOP"]) ? $row["IDENTRSAIDOP"] : null;
                $cdInvest = $row["DSORIGEM"] === 'INVESTIMENTOS' ? $row["CDCODIGO"] : null;
                $cdDespAdmFat = isset($row["CDDESPADMFAT"]) ? $row["CDDESPADMFAT"] : null;
                $cdDespPrev = $row["DSORIGEM"] === 'DESPESA' ? $row["CDCODIGO"] : null;
                $cdFilialMo = isset($row["CDFILIALMO"]) ? $row["CDFILIALMO"] : null;
                $vrPercentual = isset($row["VRPERCENTUAL"]) ? $row["VRPERCENTUAL"] : 0;
                $cdOrcamento = $row["DSORIGEM"] === 'ORCAMENTOMO' ? $row["CDCODIGO"] : null;
                $cdFatuDiver = $row["DSORIGEM"] === 'DIVERSOS' ? $row["CDCODIGO"] : null;
                $cdSubGrProd = $row["DSORIGEM"] === 'SUBGRUPOPROD' ? $row["CDCODIGO"] : null;
                $cdServFili = $row["DSORIGEM"] === 'SERVICO' ? $row["CDCODIGO"] : null;
                $cdTpcPagar = $row["DSORIGEM"] === 'TCPAGAR' ? $row["CDCODIGO"] : null;
                $cdContCtbl = $row["DSORIGEM"] === 'CONTABIL' ? $row["CDCODIGO"] : null;
                $cdGrupProd = isset($row["CDGRUPPROD"]) ? $row["CDGRUPPROD"] : null;
                $dsExpressao = isset($row["DSEXPRESSAO"]) ? $row["DSEXPRESSAO"] : null;
                if ($cdFilial === 'P') {
                    if($newRegister) {
                        $params = [
                            "NRPERCITEMGOP" => $rowItemOrca["NRSEQITEMORCA"], 
                            "CDPRODUTO" => $cdProduto,
                            "CDIMPOSTO" => $cdImposto,
                            "CDTIPOOPER" => $cdTipoOper,
                            "IDENTRSAIDOP" =>  $idEntrSaidOp,
                            "CDINVEST" => $cdInvest,
                            "CDDESPADMFAT" => $cdDespAdmFat,
                            "CDDESPPREV" => $cdDespPrev,
                            "CDFILIALMO" => $cdFilialMo,
                            "VRPERCENTUAL" => $vrPercentual,
                            "CDOPERINCLUSAO" => $operator, 
                            "CDORCAMENTOMO" => $cdOrcamento,
                            "CDFATUDIVER" => $cdFatuDiver, 
                            "CDSUBGRPROD" => $cdSubGrProd, 
                            "CDSERVFILI" => $cdServFili, 
                            "CDTPCTPAGAR" => $cdTpcPagar, 
                            "CDCONTCTBL" => $cdContCtbl,
                            "CDGRUPPROD" => $cdGrupProd, 
                            "DSEXPRESSAO" => $dsExpressao
                        ];
                        $this->service->insertDetail($params);
                    } else{
                        $params = [
                            "VRPERCENTUAL" => $row["VRPERCENTUAL"],
                            "NRPERCITEMDETALGOP" => $row["NRSEQUENCIAL"]
                        ];
                        $this->service->updateDetail($params);
                    }
                } else {
                    if($newRegister) {
                        $params = [
                            "NRPERCITFILGOP" => $rowItemOrca['NRSEQITEMOP'],
                            "CDFILIAL" => $cdFilial,
                            "CDPRODUTO" => $cdProduto,
                            "CDIMPOSTO" => $cdImposto,
                            "CDTIPOOPER" => $cdTipoOper,
                            "CDFILIALMO" => $cdFilialMo,
                            "IDENTRSAIDOP" => $idEntrSaidOp,
                            "CDINVEST" => $cdInvest,
                            "CDDESPADMFAT" => $cdDespAdmFat,
                            "CDDESPPREV" => $cdDespPrev,
                            "IDDESPPREV" => $idDespPrev,
                            "VRPERCENTUAL" => $vrPercentual,
                            "NRORG" => $nrOrg,
                            "NRORGINCLUSAO" => $nrOrg,
                            "CDOPERINCLUSAO" => $operator,
                            "CDORCAMENTOMO" => $cdOrcamento,
                            "CDFATUDIVER" => $cdFatuDiver,
                            "CDSERVFILI" => $cdServFili,
                            "CDSUBGRPROD" => $cdSubGrProd,
                            "CDTPCTPAGAR" => $cdTpcPagar,
                            "CDCONTCTBL" => $cdContCtbl,
                            "CDGRUPPROD" => $cdGrupProd,
                            "DSEXPRESSAO" => $dsExpressao
                        ];
                        $this->service->insertDetailUnit($params);
                    } else {
                        $params = [
                            "VRPERCENTUAL" => $row["VRPERCENTUAL"],
                            "NRPERCIDETALFILGOP" => $row["NRPERCIDETALFILGOP"]
                        ];
                        $this->service->updateDetailUnit($params);
                    }
                }
            }
            $this->service->commit();
        } catch (\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function updateDetailModal(Request\DataSet $request, Response $response) {
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->beginTransaction();
            $rowDetail = $rows[1];
            $cdFilial = $rowDetail["CDFILIAL"];
            foreach ($rows[0] as $row) {
                $params = [
                    "NRPERCITEMDETALGOP" => $rowDetail['NRSEQUENCIAL'],
                    "NRPERCITEMDETALFILGOP" => $rowDetail['NRSEQUENCIAL'],
                    "CDPRODUTO" => $row["TIPO"] === 'PRODUTO' ? $row["CODIGOORIG"] : null,
                    "CDORCAMENTOMO" => $row["TIPO"] === 'ORCAMENTOMO' ? $row["CODIGOORIG"] : null,
                    "CDDESPADMFAT" => isset($row["CDDESPADMFAT"]) ? $row["CDDESPADMFAT"] : null,
                    "CDTIPOOPER" => $row["TIPO"] === 'TIPOOPERAC' ? $row["CODIGOORIG"] : null,
                    "CDFILIALMO" => isset($row["CDFILIALMO"]) ? $row["CDFILIALMO"] : null,
                    "IDENTRSAIDOP" => isset($row["IDENTRSAIDOP"]) ? $row["IDENTRSAIDOP"] : null,
                    "VRPERCENTUAL" => isset($row["VRPERCENTUAL"]) ? $row["VRPERCENTUAL"] : 0,
                    "NRORG" => $nrOrg = $this->environment->getNrorg(),
                    "CDCONTCTBL" => $row["TIPO"] === 'CONTABIL' ? $row["CODIGOORIG"] : null,
                    "CDTPCTPAGAR" => $row["TIPO"] === 'TCPAGAR' ? $row["CODIGOORIG"] : null
                ];
                $paramsUpdate = [
                    "VRPERCENTUAL" => isset($row["VRPERCENTUAL"]) ? $row["VRPERCENTUAL"] : 0,
                    "NRPERCDETALPRODGOP" => $row['NRSEQ'],
                    "NRPERCDETPRODFILGOP" => $row['NRSEQ'],
                ];
                if ($cdFilial === 'P') {
                    if($row["NRSEQ"] !== 0) {
                        unset($paramsUpdate["NRPERCDETPRODFILGOP"]);
                        $this->service->updateDetailModal($paramsUpdate);
                    } else {
                        unset($params["NRPERCITEMDETALFILGOP"]);
                        $this->service->insertDetailModal($params);
                    }
                } else {
                    if($row["NRSEQ"] !== 0) {
                        unset($paramsUpdate["NRPERCDETALPRODGOP"]);
                        $this->service->updateDetailModalUnit($params);
                    } else {
                        unset($params["NRPERCITEMDETALGOP"]);
                        $this->service->insertDetailModalUnit($params);
                    }
                }
            }
            $this->service->commit();
        } catch (\Exception $ex) {
            $this->service->rollback();
            $response->setError(new Error($ex->getMessage(), 500));
        }
    }
    
    public function getFilioperPadr(Request\Filter $request, Response $response) {
        $params = ['CDOPERADOR' ];
        $conditions = $this->util->prepareConditions($params);
        array_push($conditions, array(
                "columnName" => "IDCNTRRESC",
                "operator"   => '=',
                "value"      => 'T'
            ));
        $data = $this->crudService->getFilteredDataSet('filioperPadr', $conditions)->getRows();
    	$response->addDataSet(new DataSet("filioperPadr", $data));
    }
    
    public function replicateImportParams(Request\Row $request, Response $response) {
        $row = $request->getRow();
        $sessionData = $this->util->getSessionData();
        $arrayUnits = [];
        foreach($row['CDFILDEST'] as $unit) {
            array_push($arrayUnits, $unit['CDFILIAL']);
        };
        $listFilDest = $this->util->insertLista($arrayUnits);
        
        $itemOpera = implode('_', $row['NRITEMOPERACIONAL']);
        $params = [
          'CDFILDEST'  => $listFilDest,
          'ITEMOPERA'  => '_'.$itemOpera.'_',
          'CDOPERADOR' =>  $sessionData['CDOPERADOR'],
          'CDFILORIG'  => isset($row['CDFILIAL']) ? $row['CDFILIAL'] : null
        ];
        $this->service->replicateImportParams($params);
    }
}