import {
  Metadata,
  Loader,
  Http,
  IEventParam,
  IDictionary,
  I18n,
} from '@zeedhi/core';
import {
  Button,
  Report,
  LoadingService,
  ITabsBeforeChangeEventParam,
  Number as ZdNumber,
} from '@zeedhi/common';
import BudgetParameters from '@/controllers/BudgetParameters';
import { ZdTreeGrid, ZdTabs } from '@zeedhi/vuetify';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import { OperatingBudgetConclusionController } from '@/controllers/OperatingBudgetConclusionController';

export class CalculationOperatingBudgetController {
  /*
    checkLine is responsible for identifying if there are selected lines in the main grid.

    true: exists
    false: does not exist
  */
  public checkLine = false;
  
  /*
    checkGridEdit is responsible for identifying which main grid is being edited.
  */
  public checkGridEdit = false;
  
  public checkGridDetailEdit = false;
  
  public checkDetailValueEdit = false;
  
  public checkValueEnteredInGrid = false;
  
  public checkValueEnteredInTotal = false;
  
  public valueInformedInDetails = false;
  
  public editDetail = false;
  
  public totalDetail = 0;
  
  public nrItOrcaMesGOP = null;
  
  public filter;
  
  public titleBudget: string = 'Nenhum filtro aplicado.';
  
  public unitMonthSelected: string = '';
  
  public selectedCell: { rowId: string, column: string } = { rowId: '', column: '' };
  
  public clickedCell: { row: any, column: any, component: any } = { row: null, column: null, component: null };
  
  public activeGrid;
  
  public budgetCompleted = false;
  
  public reportHeader = {
    YEAR: '',
    PERIOD: '',
    MANAGER: '',
  };
  
  private months: { [key: string]: number } = {
    JANEIRO: 1,
    FEVEREIRO: 2,
    MARCO: 3,
    ABRIL: 4,
    MAIO: 5,
    JUNHO: 6,
    JULHO: 7,
    AGOSTO: 8,
    SETEMBRO: 9,
    OUTUBRO: 10,
    NOVEMBRO: 11,
    DEZEMBRO: 12,
  };
  
  private requestOperationalItems: { [key: string]: string } = {
    gop10000_gridAdministrateTax: 'gop10000_mainGrid',
    gop10000_gridConsumption: 'gop10000_mainGrid',
    gop10000_gridDepreciation: 'gop10000_mainGrid',
    gop10000_gridEntryRelease: 'gop10000_mainGrid',
    gop10000_gridTaxRecovered: 'gop10000_mainGrid',
    gop10000_gridHeadquarters: 'gop10000_mainGrid',
    gop10000_gridInvestment: 'gop10000_mainGrid',
    gop10000_gridInvoicing: 'gop10000_mainGrid',
    gop10000_gridLabor: 'gop10000_mainGrid',
    gop10000_gridMiscellaneous: 'gop10000_mainGrid',
    gop10000_gridProfitTax: 'gop10000_mainGrid',
    gop10000_gridTurnoverTax: 'gop10000_mainGrid',
    gridSummary: 'gridSummary',
  };
  
  private mappingPage: { [key: string]: string } = {
    tabInvoicing: 'gop10000_gridInvoicing',
    tabTaxes: 'AGRUPIMP',
    tabTurnoverTax: 'gop10000_gridTurnoverTax',
    tabConsumption: 'gop10000_gridConsumption',
    tabExpense: 'AGRUPDESP',
    tabLabor: 'gop10000_gridLabor',
    tabAdministrateTax: 'gop10000_gridAdministrateTax',
    tabHeadquarters: 'gop10000_gridHeadquarters',
    tabMiscellaneous: 'gop10000_gridMiscellaneous',
    tabEntryRelease: 'gop10000_gridEntryRelease',
    tabTaxRecover: 'gop10000_gridTaxRecovered',
    tabProfitTax: 'gop10000_gridProfitTax',
    tabInvestment: 'gop10000_gridInvestment',
    tabDepreciation: 'gop10000_gridDepreciation',
  };
    
  private mappingDetail: { [key: string]: string } = {
    INVESTIMENTO: 'gop10000_gridInvestmentDetail',
    IMPOSTOSOBLUCRO: 'gop10000_gridProfitTaxDetail',
    IMPOSTOARECUPERAR: 'gop10000_gridTaxRecoveredDetail',
    IMPOSTO: 'gop10000_gridTurnoverTaxDetail',
    DESPENTRADA: 'gop10000_gridEntryReleaseDetail',
    DESPDIVERSA: 'gop10000_gridMiscellaneousDetail',
    DESPMATRIZ: 'gop10000_gridHeadquartersDetail',
    MAOOBRA: 'gop10000_gridLaborDetail',
    CONSUMO: 'gop10000_gridConsumptionDetail',
    FATURAMENTO: 'gop10000_gridInvoicingDetail',
  };
  
  public daysServ = {
    sat: 0,
    sun: 0,
    hol: 0,
    mon: 0,
    wee: 0,
    wor: 0,
  };
  
  public async mounted() {
    await BudgetParameters.getBudgetParameters();
    const filterOperatingBudgetModal = Loader.getInstance('FilterOperatingBudgetModalController');
    filterOperatingBudgetModal.createfilterOperatingBudgetModal();
    this.enableOperationalItem();
  }
  
  public defineTitleBudget() {
    let title = '';
    if (this.filter) {
      const year = this.filter.NRANOORCAGOP;
      const period = !this.filter.NRPERIODOORCA || this.filter.NRPERIODOORCA.IDPERIODOORCA === 'ANUAL'
        ? this.filter.IDPERIODOORCA : this.filter.NRPERIODOORCA.DSPERIODOORCA;
      const manager = BudgetParameters.getParameters.NMOPERADOR;
      title = I18n.translate('Ano').concat(': ', year, ' - ',
        I18n.translate('Período'), ': ', period, ' - ',
        I18n.translate('gop::Gerente Operacional'), ': ', manager);
      this.titleBudget = title;
      this.reportHeader = {
        YEAR: year,
        PERIOD: period,
        MANAGER: manager,
      };
    }
    this.initialize();
  }
  
  public async tableData(component) {
    const utilities = new Utilities();
    let element;
    
    if (!component.name) {
      element = component.component;
    } else {
      element = component;
    }
    
    const routeName = this.requestOperationalItems[element.name];
    if (routeName) {
      BudgetParameters.setRecharged(routeName);
      const filtered = BudgetParameters.getParametersHandled;
      const sendObject = {
        filter: [],
      };
      sendObject.filter.push(...filtered);
      
      let specificFilter;
      if (element.datasource.filter) {
        specificFilter = Object.entries(element.datasource.filter).map(([key, value]) => ({
          name: key,
          operator: '=',
          value,
        }));
        
        sendObject.filter.push(...specificFilter);
      }
      
      let data;
      
      LoadingService.show();
      /*
      await Http.post(`/${routeName}`, sendObject).then((response) => {
        data = response.data.dataset[routeName];
        
        if (routeName === 'gridSummary' && data.some((obj) => ('IDTIPOORCAOP' in obj))) {
          data.forEach((obj) => {
            obj.IDTIPOORCAOP = I18n.translate('gop::'.concat(obj.IDTIPOORCAOP));
          });
        }
        element.datasource.setFilter([]);
        await element.datasource.updateData([]);
        element.datasource.updateData(data);
        LoadingService.hide();
      }).catch((e) => {
        utilities.errorMessage('Falha ao buscar dados');
        LoadingService.hide();
        console.warn(e);
      });
      */
      try {
        const response = await Http.post(`/${routeName}`, sendObject);
        data = response.data.dataset[routeName];
      
        if (routeName === 'gridSummary' && data.some((obj) => ('IDTIPOORCAOP' in obj))) {
          data.forEach((obj) => {
            obj.IDTIPOORCAOP = I18n.translate('gop::'.concat(obj.IDTIPOORCAOP));
          });
        }
      
        element.datasource.setFilter([]);
        await element.datasource.updateData([]);
        await element.datasource.updateData(data);
        LoadingService.hide();
      } catch (e) {
        utilities.errorMessage('Falha ao buscar dados');
        LoadingService.hide();
        console.warn(e);
      }
    }
  }
  
  async sortMonths({ component }) {
    const startingMonth = parseInt(BudgetParameters.getParameters.NRMESINIORCA, 10);
    const visibleMonth = BudgetParameters.budgetMonths;
    let position = startingMonth - 1;
    let exclusionPosition;
    let janPosition = 0;
    
    Object.entries(this.months).forEach(([key, value]) => {
      const index = component.columns.findIndex((obj) => (obj.name === key));
      if (janPosition === 0) {
        janPosition = index;
      } else {
        janPosition += 1;
      }
      component.columns[index].isVisible = visibleMonth.includes(key);
      
      if (index !== -1 && position !== 0) {
        position += value === 1 ? index : 1;
        exclusionPosition = position + 1;
        if (component.columns[position].name !== 'TOTAL') {
          component.columns.splice(janPosition, 0, component.columns[position]);
          component.columns.splice(exclusionPosition, 1);
        } else {
          position = 0;
        }
      }
    });
    
    return component.columns;
  }
  
  public onInlineEditConditions({ component }: IEventParam<TekGrid>) {
    this.checkGridEdit = component.editing;
  }
  
  public onInlineDetailEditConditions({ component }: IEventParam<TekGrid>) {
    this.checkGridDetailEdit = component.editing;
  }
  
  public cellIsEditable({ row, column }: IDictionary<any>) {
    const month = this.months[column.name].toString().padStart(2, '0');
    let monthWithinLimit;
    let detailed = false;
    
    if (row.IDEDITMES) {
      const firstEditableMonth = parseInt(row.IDEDITMES.split('/').join(''), 10);
      const startingMonth = parseInt(BudgetParameters.getParameters.NRMESINIORCA, 10);
      const nextYear = parseInt(row.NRANOORCAGOP, 10) + 1;
      let budgetMonth;
      
      if (startingMonth > this.months[column.name]) {
        budgetMonth = parseInt(nextYear.toString().concat(month), 10);
      } else {
        budgetMonth = parseInt(row.NRANOORCAGOP.toString().concat(month), 10);
      }
      
      monthWithinLimit = (budgetMonth >= firstEditableMonth);
    } else {
      monthWithinLimit = false;
    }
    if (row.MESDETAL) {
      detailed = row.MESDETAL.split(',').includes(month);
    }
    return row.CONCLUSAO === 'PENDENTE' && !detailed && monthWithinLimit;
  }
  
  public async cancelEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    await grid.cancelEditedRows();
    this.onInlineEditConditions({ component: grid });
    await this.rechargeBudget({ component: grid });
  }
  
  public async cancelDetailEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    const gridNameMaster = grid.name.replace(/Detail$/, '');
    const gridMaster = Metadata.getInstance<TekGrid>(gridNameMaster);
    
    const cellEdit = gridMaster.datasource.data.find((obj) => (
      obj.DSFILIAL === this.clickedCell.row.DSFILIAL
    ));
    
    await grid.cancelEditedRows();
    this.onInlineDetailEditConditions({ component: grid });
    await this.rechargeBudget({ component: grid });
    this.totalDetail = cellEdit[this.clickedCell.column.name];
    
    this.checkDetailValueEdit = false;
    this.checkValueEnteredInTotal = false;
  }
  
  public async saveEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    const data = grid.getEditedRows();
    const utilities = new Utilities();
    
    LoadingService.show();
    Http.post('/changeBudgetMonth', {
      requestType: 'DataSet',
      dataset: data,
    }).then(async () => {
      await grid.saveEditedRows();
      this.onInlineEditConditions({ component: grid });
      LoadingService.hide();
      
      await this.rechargeBudget({ component: grid });
      utilities.successMessage();
    }).catch((e) => {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  public async saveDetailEditing({ component }: IEventParam<Button>) {
    const grid = component.parent as TekGrid;
    const gridNameMaster = grid.name.replace(/Detail$/, '');
    const gridMaster = Metadata.getInstance<TekGrid>(gridNameMaster);
    const data = grid.getEditedRows();
    const utilities = new Utilities();
    
    LoadingService.show();
    try {
      if (data && data.length > 0) {
        await Http.post('/changeBudgetDetailMonth', {
          requestType: 'DataSet',
          dataset: data,
        });
        this.checkValueEnteredInGrid = true;
      } else {
        let valueNameTotalDetail = grid.name.slice('gop10000_grid'.length);
        valueNameTotalDetail = valueNameTotalDetail.charAt(0).toLowerCase()
          + valueNameTotalDetail.slice(1);
        const totalDetail = Metadata.getInstance<ZdNumber>(valueNameTotalDetail.concat('Value'));
        
        const filter = {
          NRITORCAMESGOP: grid.datasource.data[0].NRITORCAMESGOP,
          VRORCAMENTO: totalDetail.value,
        };
        
        await Http.post('/changeBudgetDetMonthTotal', filter);
        this.checkValueEnteredInGrid = false;
      }
      await grid.saveEditedRows();
      this.onInlineDetailEditConditions({ component: grid });
      
      utilities.successMessage();
      this.checkDetailValueEdit = false;
      this.checkValueEnteredInTotal = false;
      
      // gridMaster.cellClick(this.clickedCell.row, this.clickedCell.column, {} as any, {} as any);
    } catch (e) {
      utilities.errorMessage();
      console.warn(e);
    }
    await this.rechargeBudget({ component: gridMaster });
      
    const cellEdit = gridMaster.datasource.data.find((obj) => (
      obj.DSFILIAL === this.clickedCell.row.DSFILIAL
    ));
    this.totalDetail = cellEdit[this.clickedCell.column.name];
    
    LoadingService.hide();
    this.checkDetailValueEdit = false;
    this.checkValueEnteredInTotal = false;
  }
  
  /*
  private budgetCalculation(data) {
    const utilities = new Utilities();
    
    LoadingService.show();
    Http.post('/budgetCalculation', data).then((response) => {
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      utilities.successMessage();
      LoadingService.hide();
      utilities.hideDialogue();
    }).catch((e) => {
      LoadingService.hide();
      utilities.errorMessage();
      console.warn(e);
    });
  }
  */
  private async budgetCalculation(data) {
    const utilities = new Utilities();
    
    try {
      LoadingService.show();
      
      const response = await Http.post('/budgetCalculation', data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      
      utilities.successMessage();
      utilities.hideDialogue();
    } catch (e) {
      utilities.errorMessage();
      console.warn(e);
    } finally {
      LoadingService.hide();
    }
  }
  
  public async confirmationServices({ component }) {
    const grid = component.parent as TekGrid;
    const utilities = new Utilities();
    
    // if (!['gop10000_gridInvoicingDetail', 'gop10000_gridTurnoverTaxDetail'].includes(grid.name)) {
    if (!Object.values(this.mappingDetail).includes(grid.name)) {
      const selectedLines = grid.selectionState.except;
      const selectedUnits = selectedLines.map((obj) => (obj.CDFILIAL));
      const regexPercentage: RegExp = /percentage/i;
      const calcByPercentage: boolean = regexPercentage.test(component.name);
      const regexConclude: RegExp = /conclude/i;
      const budgetConclude: boolean = regexConclude.test(component.name);
      const regexPrevious: RegExp = /previous/i;
      const budgetPrevious: boolean = regexPrevious.test(component.name);
      
      /*
        P: Calculation by percentage
        O: Standard budget calculation
      */
      let calculation;
      if (calcByPercentage) {
        calculation = 'P';
      } else if (budgetPrevious) {
        calculation = 'D';
      } else {
        calculation = 'O';
      }
      
      const data = {
        CDFILIAL: selectedUnits,
        CALCULATION: calculation,
        NRANOORCAGOP: selectedLines[0].NRANOORCAGOP,
        IDPERIODOORCA: selectedLines[0].IDPERIODOORCA,
        NRPERIODOORCA: selectedLines[0].NRPERIODOORCA,
        IDTIPOORCAOP: selectedLines[0].IDTIPOORCAOP,
        MESSAGE: '',
      };
      
      if (calcByPercentage) {
        data.MESSAGE = 'gop::CALC_ORCA_PERC';
      } else if (budgetConclude) {
        data.MESSAGE = I18n.translate('gop::CONC_ORCAMENTO');
      } else if (budgetPrevious) {
        data.MESSAGE = 'gop::CALC_ORCA_OLDORCA';
      } else {
        data.MESSAGE = '';
      }
      if (budgetConclude) {
        const budgetConcludeController = new OperatingBudgetConclusionController();
        const confirmed = await utilities.showDialog(budgetConcludeController.completeBudget, data, 'YN');
        if (confirmed === 'Yes') {
          await this.rechargeBudget({ component: grid });
          await grid.selectAll(false);
          this.checkLine = false;
        }
      } else {
        const confirmed = await utilities.showDialog(this.budgetCalculation, data, 'YN');
        if (confirmed === 'Yes') {
          await this.rechargeBudget({ component: grid });
          await grid.selectAll(false);
          this.checkLine = false;
        }
      }
    } else {
      const regexClearDetail: RegExp = /clear/i;
      const budgetClearDetail: boolean = regexClearDetail.test(component.name);
      
      const data = {
        NRITORCAMESGOP: this.nrItOrcaMesGOP,
        MESSAGE: '',
        GRID: grid.name,
      };
      if (budgetClearDetail) {
        data.MESSAGE = 'gop::CLEAN_DETAILING';
        await utilities.showDialog(this.cleanDetailing, data, 'YN');
      }
    }
  }
  
  // public rechargeBudget = async ({ component }) => {
  public rechargeBudget = async ({ component }) => new Promise<void>((resolve, reject) => {
    try {
      let grid;
      if (component.component === 'TekGrid') {
        grid = component;
      } else {
        grid = component.parent as TekGrid;
      }
      let filter;
      
      switch (grid.name) {
        case 'gop10000_gridAdministrateTax':
          filter = { IDTIPOORCAOP: 'TAXAADM' };
          break;
        case 'gop10000_gridConsumption':
          filter = { IDTIPOORCAOP: 'CONSUMO' };
          break;
        case 'gop10000_gridDepreciation':
          filter = { IDTIPOORCAOP: 'DEPRECIACAO' };
          break;
        case 'gop10000_gridEntryRelease':
          filter = { IDTIPOORCAOP: 'DESPENTRADA' };
          break;
        case 'gop10000_gridTaxRecovered':
          filter = { IDTIPOORCAOP: 'IMPOSTOARECUPERAR' };
          break;
        case 'gop10000_gridHeadquarters':
          filter = { IDTIPOORCAOP: 'DESPMATRIZ' };
          break;
        case 'gop10000_gridInvestment':
          filter = { IDTIPOORCAOP: 'INVESTIMENTO' };
          break;
        case 'gop10000_gridInvoicing':
          filter = { IDTIPOORCAOP: 'FATURAMENTO' };
          break;
        case 'gop10000_gridLabor':
          filter = { IDTIPOORCAOP: 'MAOOBRA' };
          break;
        case 'gop10000_gridMiscellaneous':
          filter = { IDTIPOORCAOP: 'DESPDIVERSA' };
          break;
        case 'gop10000_gridProfitTax':
          filter = { IDTIPOORCAOP: 'IMPOSTOSOBLUCRO' };
          break;
        case 'gop10000_gridTurnoverTax':
          filter = { IDTIPOORCAOP: 'IMPOSTO' };
          break;
        default:
          filter = '';
          break;
      }
      
      grid.datasource.filter = filter;
      this.tableData(grid);
      // };
      resolve();
    } catch (error) {
      reject(error);
    }
  });
  
  public initialize() {
    const instance = Metadata.getInstance<ZdTreeGrid>('gridSummary');
    this.tableData(instance);
  }

  get disabledButtons() {
    return this.checkGridEdit || !this.checkLine;
  }
  
  get disabledPages() {
    return this.checkGridEdit || this.checkGridDetailEdit || this.checkDetailValueEdit || this.checkValueEnteredInTotal;
  }

  get disabledRecharge() {
    return this.checkGridEdit;
  }

  get disabledRepeat() {
    return this.checkGridEdit || !this.disabledButtons;
  }
  
  public disableSelection({ row }: IDictionary) {
    return row.CONCLUSAO !== 'PENDENTE';
  }

  public rowStyleConditions(row: IDictionary) {
    let lineColor = '';
    
    if (row.IDAPROVACAOGOP in ['REPROVADO', 'CANCELADO']) {
      lineColor = '#fffee1';
    }
    // ffdece vermelho
    // dceafc azul
    // cff6c8 verde efffe9
    // rosa fff0f6
    // amarelo fffee1
    return {
      cssStyle: { background: lineColor },
    };
  }
  
  public selectedLine({ component }) {
    this.checkLine = (component.selectionState.except.length > 0);
  }
  
  public unselectedLine({ component }) {
    this.checkLine = (component.selectionState.except.length > 0);
  }
  
  public enableOperationalItem() {
    const instance = Metadata.getInstance<ZdTabs>('tabsBudget');
    const itemOperaList = BudgetParameters.getParameters.ITEMOPERACIONAL;
    const mapping: { [key: string]: string } = {
      tabSummary: 'RESUMO',
      tabInvoicing: 'FATURAMENTO',
      tabTaxes: 'AGRUPIMP',
      tabTurnoverTax: 'IMPOSTO',
      tabConsumption: 'CONSUMO',
      tabExpense: 'AGRUPDESP',
      tabLabor: 'MAOOBRA',
      tabAdministrateTax: 'TAXAADM',
      tabHeadquarters: 'DESPMATRIZ',
      tabMiscellaneous: 'DESPDIVERSA',
      tabEntryRelease: 'DESPENTRADA',
      tabTaxRecover: 'IMPOSTOARECUPERAR',
      tabProfitTax: 'IMPOSTOSOBLUCRO',
      tabInvestment: 'INVESTIMENTO',
      tabDepreciation: 'DEPRECIACAO',
    };
    let daughterTab;
    instance.tabs.forEach((item) => {
      if (Array.isArray(item.children[0].tabs)) {
        daughterTab = false;
        item.children[0].tabs.forEach((subItem) => {
          subItem.isVisible = itemOperaList.includes(mapping[subItem.name]);
          if (!daughterTab) {
            daughterTab = subItem.isVisible;
          }
        });
        item.isVisible = daughterTab;
      } else {
        item.isVisible = itemOperaList.includes(mapping[item.name]);
        if (item.name === 'tabSummary') {
          item.isVisible = itemOperaList.includes(',');
        }
      }
    });
  }
  
  /**
  * Budget observations, a function designed to inform the current status of an operational
  * item in the budget of a given unit.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public budgetObservation({ row }: IEventParam<Button>) {
    const utilities = new Utilities();
    const data = {
      TITLE: I18n.translate('Unidade').concat(`: ${row.CDFILIAL} - ${row.NMFILIAL}`),
      MESSAGE: '',
    };
    
    switch (row.IDAPROVACAOGOP) {
      case 'PENDENTE':
        data.MESSAGE = I18n.translate('gop::PENDING_BUDGET_APPROVAL');
        break;
      case 'APROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_APPROVED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'REPROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_REJECTED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'CANCELADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_CANCELED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      default:
        data.MESSAGE = row.CONCLUSAO === 'PENDENTE'
          ? I18n.translate('gop::PENDING_BUDGET_CONC') : I18n.translate('gop::PENDING_APPROVAL');
        break;
    }
    return utilities.showDialog(null, data, 'OK');
  }
  
  /**
  * Standard model for exporting reports using zeedhi, in pdf and xls formats.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public async exportReport({ component }: IEventParam<Button>) {
    const grid = component.parent.parent as TekGrid;
    let title;
    
    switch (grid.name) {
      case 'gop10000_gridAdministrateTax':
        title = 'T_TAXAADM';
        break;
      case 'gop10000_gridConsumption':
        title = 'T_CONSUMO';
        break;
      case 'gop10000_gridDepreciation':
        title = 'T_DEPRECIACAO';
        break;
      case 'gop10000_gridEntryRelease':
        title = 'T_DESPENTRADA';
        break;
      case 'gop10000_gridTaxRecovered':
        title = 'T_IMPOSTOARECUPERAR';
        break;
      case 'gop10000_gridHeadquarters':
        title = 'T_DESPMATRIZ';
        break;
      case 'gop10000_gridInvestment':
        title = 'T_INVESTIMENTO';
        break;
      case 'gop10000_gridInvoicing':
        title = 'T_FATURAMENTO';
        break;
      case 'gop10000_gridLabor':
        title = 'T_MAOOBRA';
        break;
      case 'gop10000_gridMiscellaneous':
        title = 'T_DESPDIVERSA';
        break;
      case 'gop10000_gridProfitTax':
        title = 'T_IMPOSTOSOBLUCRO';
        break;
      case 'gop10000_gridTurnoverTax':
        title = 'T_IMPOSTO';
        break;
      default:
        title = '';
        break;
    }
    const report = new Report(grid, I18n.translate('gop::'.concat(title)));
    
    const reportHeader = {
      filter: [
        { label: I18n.translate('Ano'), value: this.reportHeader.YEAR },
        { label: I18n.translate('Período'), value: this.reportHeader.PERIOD },
        { label: I18n.translate('Gerente Operacional'), value: this.reportHeader.MANAGER },
      ],
    };
    
    const rowObj = {
      dataSet: {},
      filter: {},
      metaData: reportHeader,
    };

    const reportUrl = await report.getReport(component.label.toLowerCase(), false, rowObj);
    window.open(reportUrl);
  }
  
  public async updateDataNecessary({ component, tabIndex }: ITabsBeforeChangeEventParam) {
    const tab = component.tabs[tabIndex].name;
    this.unitMonthSelected = '';
    
    this.clickedCell.column = null;
    this.clickedCell.row = null;
    
    const tooltipImport = Metadata.getInstance<Button>('spreadsheetImportButton');
    const tooltipDownload = Metadata.getInstance<Button>('spreadsheetDownloadButton');
    tooltipImport.isVisible = tab === 'tabSummary';
    tooltipDownload.isVisible = tab === 'tabSummary';
    
    if (tab === 'tabSummary') {
      this.initialize();
    }
    
    /*
    if (this.mappingPage[tab] && BudgetParameters.getNotRecharged(this.mappingPage[tab])) {
      const grid = Metadata.getInstance<TekGrid>(this.mappingPage[tab]);
      await this.rechargeBudget({ component: grid });
      
      grid.parent.parent.children[0].height = '99%';
        
      grid.datasource.setFilter([]);
      await grid.cancelEditedRows();
      await grid.datasource.updateData([]);
      
      const gridDetail = Metadata.getInstance<TekGrid>(this.mappingPage[tab].concat('Detail'));
      gridDetail.parent.isVisible = false;
      gridDetail.datasource.setFilter([]);
      gridDetail.datasource.updateData([]);
    }
    */
  }

  public async clickRepeatButtons({ component }) {
    const utilities = new Utilities();
    const data = {
      CDFILIAL: null,
      TITLE: I18n.translate('gop::REPLICATE_VALUES'),
      NRANOORCAGOP: null,
      IDTIPOORCAOP: null,
      MES: null,
      FIXEDVALUE: null,
      MESSAGE: '',
    };
    
    if (this.clickedCell.row && this.clickedCell.column) {
      data.CDFILIAL = this.clickedCell.row.CDFILIAL;
      data.NRANOORCAGOP = this.clickedCell.row.NRANOORCAGOP;
      data.IDTIPOORCAOP = this.clickedCell.row.IDTIPOORCAOP;
      data.MES = this.months[this.clickedCell.column.name];
      data.FIXEDVALUE = component.name === 'btnRepeatFixedValue' ? 'Y' : 'N';
      
      data.MESSAGE = data.FIXEDVALUE === 'Y'
        ? I18n.translate('gop::REPLICATE_FIXED_VALUE')
        : I18n.translate('gop::REPLICATE_VALUES_BUSINESS');
      
      const confirmed = await utilities.showDialog(this.repeatBudget, data, 'YN');
      if (confirmed === 'Yes') {
        const grid = component.parent as TekGrid;
        await this.rechargeBudget({ component: grid });
      }
    } else {
      data.MESSAGE = I18n.translate('gop::CALC_ORCA_REPLICATE_VALUES');
      utilities.showDialog(null, data, 'OK');
    }
    // this.repeatValue = !this.repeatValue;
    // localStorage.setItem('repeatValueBudget_GOP', this.repeatValue ? 'true' : 'false');
    
    // if (!['gop10000_gridInvoicingDetail', 'gop10000_gridTurnoverTaxDetail'].includes(grid.name)) {
    /*
    if (!Object.values(this.mappingDetail).includes(grid.name)) {
      const selectedLines = grid.selectionState.except;
      const selectedUnits = selectedLines.map((obj) => (obj.CDFILIAL));
      const regexPercentage: RegExp = /percentage/i;
      const calcByPercentage: boolean = regexPercentage.test(component.name);
      const regexConclude: RegExp = /conclude/i;
      const budgetConclude: boolean = regexConclude.test(component.name);
      const regexPrevious: RegExp = /previous/i;
      const budgetPrevious: boolean = regexPrevious.test(component.name);
      
      /*
        P: Calculation by percentage
        O: Standard budget calculation
      */
    /*
    let calculation;
      if (calcByPercentage) {
        calculation = 'P';
      } else if (budgetPrevious) {
        calculation = 'D';
      } else {
        calculation = 'O';
      }
      
      const data = {
        CDFILIAL: selectedUnits,
        CALCULATION: calculation,
        NRANOORCAGOP: selectedLines[0].NRANOORCAGOP,
        IDPERIODOORCA: selectedLines[0].IDPERIODOORCA,
        NRPERIODOORCA: selectedLines[0].NRPERIODOORCA,
        IDTIPOORCAOP: selectedLines[0].IDTIPOORCAOP,
        MESSAGE: '',
      };
      
      if (calcByPercentage) {
        data.MESSAGE = 'gop::CALC_ORCA_PERC';
      } else if (budgetConclude) {
        data.MESSAGE = I18n.translate('gop::CONC_ORCAMENTO');
      } else if (budgetPrevious) {
        data.MESSAGE = 'gop::CALC_ORCA_OLDORCA';
      } else {
        data.MESSAGE = '';
      }
      if (budgetConclude) {
        const budgetConcludeController = new OperatingBudgetConclusionController();
        const confirmed = await utilities.showDialog(budgetConcludeController.completeBudget, data, 'YN');
        if (confirmed === 'Yes') {
          await this.rechargeBudget({ component: grid });
          await grid.selectAll(false);
          this.checkLine = false;
        }
      } else {
        const confirmed = await utilities.showDialog(this.budgetCalculation, data, 'YN');
        if (confirmed === 'Yes') {
          await this.rechargeBudget({ component: grid });
          await grid.selectAll(false);
          this.checkLine = false;
        }
      }
    } else {
      const regexClearDetail: RegExp = /clear/i;
      const budgetClearDetail: boolean = regexClearDetail.test(component.name);
      
      const data = {
        NRITORCAMESGOP: this.nrItOrcaMesGOP,
        MESSAGE: '',
        GRID: grid.name,
      };
      if (budgetClearDetail) {
        data.MESSAGE = 'gop::CLEAN_DETAILING';
        await utilities.showDialog(this.cleanDetailing, data, 'YN');
      }
    }
    */
  }
  
  public async repeatBudget(data): Promise<void> {
    const utilities = new Utilities();
    
    try {
      LoadingService.show();
      
      const response = await Http.post('/repeatBudget', data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      
      utilities.successMessage();
      utilities.hideDialogue();
    } catch (e) {
      utilities.errorMessage();
      console.warn(e);
    } finally {
      LoadingService.hide();
    }
  }

  /*
  get statusRepeatButtons() {
    return this.repeatValue;
  }

  get statusOffRepeatButtons() {
    return !this.repeatValue;
  }
  */
  
  /*
  public async repeatBudget({ component, row }: IEventParam<ZdNumber>) {
    if (this.repeatValue) {
      const grid = component.parent as TekGrid;
      const { uniqueKey } = grid.datasource;
      const rowKey = row[uniqueKey];
      const month = component.name.split('_')[0];
      const nrOrcamentoGOP = grid.datasource.currentRow.NRORCAMENTOGOP;
      const dataRow = grid.getEditedRows().find((obj) => (obj.NRORCAMENTOGOP === nrOrcamentoGOP));
      const editedValue = dataRow[month];
      
      const sendObject = {
        MES: this.months[month],
        CDFILIAL: dataRow.CDFILIAL,
        EDITEDVALUE: editedValue,
        ANO: dataRow.NRANOORCAGOP,
      };
      LoadingService.show();
      
      Http.post('/workingDays', {
        requestType: 'Row',
        row: sendObject,
      }).then(async (response) => {
        const data = response.data.dataset.workingDays.dataset;
        
        const initialIndex = grid.columns.findIndex((obj) => obj.name === month) + 1;
        const endIndex = grid.columns.findIndex((obj) => obj.name === 'TOTAL');
        
        let value = 0;
        
        if (initialIndex !== -1 && endIndex !== -1) {
          for (let i = initialIndex; i <= endIndex; i += 1) {
            const obj = grid.columns[i];
  
            if (obj.editable) {
              value = data.find((item) => (item.MES === this.months[grid.columns[i].name])).VALOR;
              grid.updateCell(rowKey, grid.columns[i].name, value);
            }
          }
        }
        component.setFocus();
        LoadingService.hide();
      }).catch((e) => {
        LoadingService.hide();
        console.warn(e);
      });
    }
  }
  */
  
  public getConditionalCellMark({ row, column }: any) {
    if (row.CDFILIAL !== this.selectedCell.rowId || column.name !== this.selectedCell.column) return '';

    return 'selected-cell';
  }
  
  public getConditionalLineMark({ row }: any) {
    if (row.CDFILIAL !== this.selectedCell.rowId) return '';
    // const grid = component.parent as TekGrid;
    // if (row.CDFILIAL !== grid.datasource.currentRow.CDFILIAL) return '';

    return '';
  }
  
  public clickedCellMark({ component, row, column }: IEventParam<TekGrid>) {
    /*
    if (component.datasource.currentRow[component.currentColumn.name] !== null
      && component.datasource.currentRow[component.currentColumn.name] !== undefined) {
      this.clickedCell.component = component;
    }
    */
    this.clickedCell.column = column;
    this.clickedCell.row = row;
    
    this.activeGrid = component.name;
    this.editDetail = false;
    this.budgetCompleted = row.CONCLUSAO !== 'PENDENTE';
    if (column.name in this.months) {
      this.unitMonthSelected = row.CDFILIAL.concat(' - ', row.NMFILIAL, ' | ', I18n.translate(column.name));
    } else {
      this.unitMonthSelected = '';
    }
    
    if (row && column && this.months[column.name]) {
      /*
      if (row[column.name] !== null && row[column.name] !== undefined) {
        this.clickedCell.component = component;
        this.clickedCell.column = column;
        this.clickedCell.row = row;
      }
      */
      const typeOfBudget = row.IDTIPOORCAOP;
      
      if (row.MESDETAL
        && row.MESDETAL.split(',').includes(this.months[column.name].toString().padStart(2, '0'))) {
        const budgetNumber = row.NRORCAMENTOGOP;
        const budgetDate = this.months[column.name];
        
        this.editDetail = row.CONCLUSAO === 'PENDENTE';
        this.tableDataDetail(budgetNumber, typeOfBudget, budgetDate, row[column.name]);
      }
      /*
      else {
        this.tableDataDetail(
          null,
          typeOfBudget,
          null,
          row[column.name] ? row[column.name] : this.clickedCell.component.datasource.data.find(
            (item) => (item.NRORCAMENTOGOP === row.NRORCAMENTOGOP),
          )[column.name],
        );
      }
      */
    }
  }
  
  public async reapplyCondition({ component }: IEventParam<TekGrid>) {
    component.datasource.data.forEach((gridRow) => component.reapplyConditions(gridRow));
  }

  /**
  * Function responsible for collecting data from detail grids directly related to the main grid.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public async tableDataDetail(budgetNumber, typeOfBudget, budgetDate, budgetValue) {
    const utilities = new Utilities();
    const routeName = 'gop10000_gridDetail';
    
    /*
    const mapping: { [key: string]: string } = {
      INVESTIMENTO: 'gop10000_gridInvestmentDetail',
      IMPOSTOSOBLUCRO: 'gop10000_gridProfitTaxDetail',
      IMPOSTOARECUPERAR: 'gop10000_gridTaxRecoveredDetail',
      IMPOSTO: 'gop10000_gridTurnoverTaxDetail',
      DESPENTRADA: 'gop10000_gridEntryReleaseDetail',
      DESPDIVERSA: 'gop10000_gridMiscellaneousDetail',
      DESPMATRIZ: 'gop10000_gridHeadquartersDetail',
      MAOOBRA: 'gop10000_gridLaborDetail',
      CONSUMO: 'gop10000_gridConsumptionDetail',
      FATURAMENTO: 'gop10000_gridInvoicingDetail',
    };
    */
    
    if (this.mappingDetail[typeOfBudget]) {
      const grid = Metadata.getInstance<TekGrid>(this.mappingDetail[typeOfBudget]);
      if (grid && budgetNumber) {
        const sendObject = {
          filter: [{
            name: 'NRORCAMENTOGOP',
            operator: '=',
            value: budgetNumber,
          },
          {
            name: 'IDTIPOORCAOP',
            operator: '=',
            value: typeOfBudget,
          },
          {
            name: 'DTORCAMENTO',
            operator: '=',
            value: budgetDate,
          }],
        };
        
        let data;
        
        LoadingService.show();
        try {
          const response = await Http.post(`/${routeName}`, sendObject);
          data = response.data.dataset[routeName];
          this.totalDetail = 0;
          this.nrItOrcaMesGOP = null;
          
          if (data.some((obj) => ('IDTIPO' in obj))) {
            data.forEach((obj) => {
              obj.IDTIPO = I18n.translate('gop::T_'.concat(obj.IDTIPO));
              this.totalDetail += obj.VRORCAMENTO;
              
              if (['FATUDIVER', 'PRODUTO', 'SUBGRPROD'].includes(obj.IDTIPOORIG)) {
                obj.QTORCAMENTO = null;
              }
              if (!this.nrItOrcaMesGOP) {
                this.nrItOrcaMesGOP = obj.NRITORCAMESGOP;
              }
            });
          }
          this.checkValueEnteredInGrid = this.totalDetail !== 0;
          if (this.totalDetail === 0) {
            this.totalDetail = budgetValue;
          }
          grid.datasource.setFilter([]);
          
          await grid.cancelEditedRows();
          await grid.datasource.updateData([]);
          await grid.datasource.updateData(data);
          grid.parent.isVisible = true;
          grid.parent.parent.children[0].height = '50%';
          
          LoadingService.hide();
        } catch (e) {
          utilities.errorMessage(I18n.translate('gop::FAILED_DATA'));
          LoadingService.hide();
          console.warn(e);
        }
        
        this.workingDaysForService(sendObject);
      } else {
        /* Detailing Grid */
        grid.parent.isVisible = false;
        grid.parent.parent.children[0].height = '99%';
        
        grid.datasource.setFilter([]);
        await grid.cancelEditedRows();
        await grid.datasource.updateData([]);
      }
    }
  }
  
  public budgetDismemberment({ row }: IEventParam<Button>) {
    const utilities = new Utilities();
    const data = {
      TITLE: I18n.translate('Unidade').concat(`: ${row.CDFILIAL} - ${row.NMFILIAL}`),
      MESSAGE: '',
    };
    
    switch (row.IDAPROVACAOGOP) {
      case 'PENDENTE':
        data.MESSAGE = I18n.translate('gop::PENDING_BUDGET_APPROVAL');
        break;
      case 'APROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_APPROVED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'REPROVADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_REJECTED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      case 'CANCELADO':
        data.MESSAGE = I18n.translate('gop::BUDGET_CANCELED').concat(`: ${row.DSOBSERVACAO}`);
        break;
      default:
        data.MESSAGE = row.CONCLUSAO === 'PENDENTE'
          ? I18n.translate('gop::PENDING_BUDGET_CONC') : I18n.translate('gop::PENDING_APPROVAL');
        break;
    }
    return utilities.showDialog(null, data, 'OK');
  }
  
  /**
  *
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public visibleDismemberment({ row }: any) {
    const visibleList: string[] = ['SERVFILI'];
    return visibleList.includes(row.IDTIPOORIG);
  }
  
  /**
  *
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public disabledDismemberment({ row }: any) {
    // const disabledList: string[] = ['PRODUTO', 'SUBGRPROD'];
    // return disabledList.includes(row.IDTIPOORIG)
    // || ((row.VRORCAMENTO !== 0) && (row.VRCUSTPADR !== 0));
    // && !row.VRCUSTPADR && (row.VRORCAMENTO !== 0);
    return !((row.VRCUSTPADR !== 0)
      || ((row.VRORCAMENTO === null || row.VRORCAMENTO === 0) && row.VRCUSTPADR === 0));
  }
  
  public workingDaysForService(sendObject) {
    const filter = sendObject.filter.find((type) => (type.name === 'IDTIPOORCAOP'));
    const routeName = 'daysService';

    if (filter) {
      if (['FATURAMENTO', 'CONSUMO'].includes(filter.value)) {
        Http.post(`/${routeName}`, sendObject).then((response) => {
          const data = response.data.dataset[routeName][0];
          
          this.daysServ.wor = data.DIASMES - data.FERIADO;
          this.daysServ.wee = data.DIASSEMANA;
          this.daysServ.sat = data.SABADO;
          this.daysServ.sun = data.DOMINGO;
          this.daysServ.hol = data.FERIADO;
          this.daysServ.mon = data.DIASMES;
        }).catch((e) => {
          console.warn(e);
        });
      }
    }
  }

  get daysServWor() {
    return I18n.translate('Dias úteis').concat(': ', this.daysServ.wor.toString());
  }

  get daysServWee() {
    return I18n.translate('Dias de Semana').concat(': ', this.daysServ.wee.toString());
  }

  get daysServSat() {
    return I18n.translate('Sábado').concat(': ', this.daysServ.sat.toString());
  }

  get daysServSun() {
    return I18n.translate('Domingo').concat(': ', this.daysServ.sun.toString());
  }

  get daysServHol() {
    return I18n.translate('Feriados').concat(': ', this.daysServ.hol.toString());
  }

  get daysServMon() {
    return I18n.translate('Dias do Mês').concat(': ', this.daysServ.mon.toString());
  }
  
  public cellIsEditableDetail({ row, column }: IDictionary<any>) {
    let aux = false;
    if (['FATUDIVER', 'PRODUTO', 'SUBGRPROD', 'IMPOSTO', 'CONTABIL', 'INVESTIMENTO'].includes(row.IDTIPOORIG)) {
      aux = column.name === 'VRORCAMENTO' && this.editDetail && !row.VRCUSTPADR;
    } else if (row.IDTIPOORIG === 'SERVFILI') {
      aux = this.editDetail && !row.VRCUSTPADR;
    }
    return aux;
  }
  
  /*
  public updateDetailValue() {
    this.checkGridDetailEdit = true;
    this.checkDetailValueEdit = true;
  }
  */
  /*
  public updateDetailValue() {
    const numEdit = Metadata.getInstance<ZdNumber>('invoicingDetailValue');
    numEdit.value = this.totalDetail;
  }
  */
  
  public updateDetailValue() {
    this.checkValueEnteredInTotal = true;
  }
  
  get valueDetails() {
    return this.checkValueEnteredInGrid || this.checkGridDetailEdit || this.budgetCompleted;
  }
  
  get valueGridDetails() {
    return this.totalDetail === 0 || this.checkValueEnteredInGrid;
  }
  
  get enableClearDetailValue() {
    return this.totalDetail === 0 || this.breakdownOfDetailEdit;
  }
  
  get detailingInEditing() {
    return this.checkGridDetailEdit || this.checkValueEnteredInTotal;
  }
  
  get breakdownOfDetailEdit() {
    return !this.editDetail;
  }
  
  public cleanDetailing = async (data) => {
    const utilities = new Utilities();
    const gridNameMaster = data.GRID.replace(/Detail$/, '');
    const gridMaster = Metadata.getInstance<TekGrid>(gridNameMaster);
    
    LoadingService.show();
    
    try {
      const response = await Http.post('/cleanDetailing', data);
      
      if (response.data.error) {
        throw new Error(response.data.error);
      }
      
      await this.rechargeBudget({ component: gridMaster });
      this.clickedCell.row = gridMaster.datasource.data.find((obj) => (obj.DSFILIAL === this.clickedCell.row.DSFILIAL));
      await gridMaster.cellClick(this.clickedCell.row, this.clickedCell.column, {} as any, {} as any);

      utilities.successMessage();
      utilities.hideDialogue();
      LoadingService.hide();
    } catch (e) {
      utilities.errorMessage();
      LoadingService.hide();
      console.warn(e);
    }
  };
  
  get cellSelected() {
    return this.unitMonthSelected !== '';
  }
  
  public async calculatesTotalDetail({ component, row, column }: IEventParam<ZdNumber>) {
    const grid = component.parent as TekGrid;
    const { uniqueKey } = grid.datasource;
    const rowKey = row[uniqueKey];
    const value = component.value;
    let toUpdate = false;

    let totalValue = 0;
    
    if (column.name === 'QTORCAMENTO' && row.QTORCAMENTO !== value) {
      toUpdate = true;
      row.QTORCAMENTO = value;
    } else if (column.name === 'VRPRECOBASE' && row.VRPRECOBASE !== value) {
      toUpdate = true;
      row.VRPRECOBASE = value;
    } else if ((column.name === 'VRORCAMENTO')
      && (row.VRPRECOBASE * row.QTORCAMENTO) !== value) {
      toUpdate = false;
      if (grid.columns.some((col) => (col.name === 'QTORCAMENTO'))) {
        grid.updateCell(rowKey, 'QTORCAMENTO', null);
        grid.updateCell(rowKey, 'VRPRECOBASE', null);
      }
      row.QTORCAMENTO = null;
      row.VRPRECOBASE = null;
    }
    if (row.QTORCAMENTO && row.VRPRECOBASE) {
      totalValue = row.VRPRECOBASE * row.QTORCAMENTO;
    }
    
    if (toUpdate) {
      grid.updateCell(rowKey, 'VRORCAMENTO', totalValue);
      row.VRORCAMENTO = totalValue;
    }
  }
  
  public clickedCellInline({ component }) {
    this.clickedCell.column = component.parent.currentColumn;
    this.clickedCell.row = component.parent.datasource.currentRow;
    
    if (this.clickedCell.column.name in this.months) {
      this.unitMonthSelected = this.clickedCell.row.CDFILIAL.concat(' - ',
        this.clickedCell.row.NMFILIAL, ' | ', I18n.translate(this.clickedCell.column.name));
    } else {
      this.unitMonthSelected = '';
    }
  }
}
