import {
  IEventParam,
  Http,
  Metadata,
  Loader,
} from '@zeedhi/core';
import { LoadingService } from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';
import { Utilities } from '@/util/Utilities';
import BudgetParameters from '@/controllers/BudgetParameters';

export class OperationalBudgetApprovalController {
  private grid!: TekGrid;
  
  private filter;
  
  private orcamentos;
  
  public async mounted() {
    await BudgetParameters.getBudgetParameters();
    const filterOperatingBudgetModal = Loader.getInstance('FilterOperatingBudgetModalController');
    filterOperatingBudgetModal.createfilterOperatingBudgetModal('operationalBudgetApproval');
  }

  public async initializeFilter() {
    const utilities = new Utilities();
    const componentGridBudgetApproval = Metadata.getInstance<TekGrid>('gridOperationalBudgetApproval');
    const filtered = BudgetParameters.getParametersHandled;
    const sendObject = {
      filter: [],
    };
    sendObject.filter.push(...filtered);
    LoadingService.show();
    await Http.post('/approvalBudgetGop', sendObject).then((response) => {
      this.orcamentos = response.data.dataset.approvalBudgetGop;
      componentGridBudgetApproval.datasource.updateData(this.orcamentos);
      LoadingService.hide();
    }).catch((e) => {
      utilities.errorMessage('Falha ao buscar dados');
      LoadingService.hide();
      console.warn(e);
    });
  }
  
  private callDescriptionModal(nameComponent, seletedItems) {
    const approvalBudgetModalController = Loader.getInstance('ApprovalBudgetModalController');
    approvalBudgetModalController.createApprovalBudgetModal(nameComponent, seletedItems);
  }
  
  public rulesToShowModal({ component }) {
    const selectedLines = this.grid.selectedRows;
    const nameComponent = component.name;
    
    if (nameComponent === 'approveButtonOperational' || nameComponent === 'reproveButtonOperational') {
      const pendingItems = selectedLines.filter((item) => item.IDAPROVACAOGOP === 'PENDENTE');
      if (pendingItems.length === 0) {
        this.showWarningDialog(nameComponent);
      } else {
        this.callDescriptionModal(nameComponent, pendingItems);
      }
    } else if (nameComponent === 'cancelButtonOperational') {
      const approvedItems = selectedLines.filter((item) => item.IDAPROVACAOGOP === 'APROVADO');
      if (approvedItems.length === 0) {
        this.showWarningDialog(nameComponent);
      } else {
        this.callDescriptionModal(nameComponent, approvedItems);
      }
    }
  }
  
  private showWarningDialog(nameComponent) {
    const utilities = new Utilities();
    let dialogMessage;
    if (nameComponent === 'approveButtonOperational') {
      dialogMessage = 'gop::ALERT_APPROVED_BUDGET';
    } else if (nameComponent === 'reproveButtonOperational') {
      dialogMessage = 'gop::ALERT_APPROVED_BUDGET';
    } else {
      dialogMessage = 'gop::ALERT_PENDING_BUDGET';
    }
    const dataForm = {
      TITLE: 'Aviso',
      MESSAGE: dialogMessage,
    };
    utilities.showDialog('Aviso', dataForm, 'ok');
  }

  public approve() {
    this.grid.saveEditedRows();
  }

  public disapprove() {
    this.grid.cancelEditedRows();
  }

  public getPdfReport() {
    this.grid.getReport('pdf', true);
  }

  public onCreated({ component }: IEventParam<TekGrid>) {
    this.grid = component;
  }
  
  public get showSearchInput() {
    return this.grid.showSearch;
  }

  public get showReloadButton() {
    return this.grid.showReload;
  }
  
  public get showDivider1() {
    return (this.showReloadButton
        || this.showLayoutOptionsButton
        || this.showColumnsButton
        || this.showActionsButton
        || this.showExportButton
        || this.showSearchInput
    );
  }
  
  public get showDivider2() {
    return (
      this.showReloadButton
      && (this.showLayoutOptionsButton
        || this.showColumnsButton
        || this.showActionsButton
        || this.showExportButton
        || this.showSearchInput)
    );
  }

  public get showColumnsButton() {
    return this.grid.columnsButton;
  }

  public get columnsButtonIgnore() {
    return this.grid.columnsButtonIgnore;
  }

  public get showLayoutOptionsButton() {
    return this.grid.showLayoutOptions;
  }

  public get showActionsButton() {
    return this.grid.actions.length > 0;
  }

  public get showExportButton() {
    return this.grid.showExport;
  }

  public get showActionAndExportButton() {
    return (
      (this.showLayoutOptionsButton || this.showColumnsButton)
      && (this.showActionsButton || this.showExportButton || this.showSearchInput)
    );
  }
  
  public get disableButton() {
    return this.grid.selectedRows.length === 0;
  }

  public get isEditing() {
    return this.grid.editing;
  }

  public get showEditButtons() {
    return this.isEditing && this.grid.showEditButtons;
  }

  public get isNotEditing() {
    return !this.isEditing;
  }

  public async reloadGrid() {
    const filterOperatingBudgetModal = Loader.getInstance('FilterOperatingBudgetModalController');
    await filterOperatingBudgetModal.applyFilter();
    this.grid.selectedRows = [];
  }
}
