import { Metadata, Loader } from '@zeedhi/core';
import {
  Modal,
  ModalService,
  Form,
  SelectMultiple,
  Select,
} from '@zeedhi/common';
import BudgetParameters from '@/controllers/BudgetParameters';
import { ZdTabs } from '@zeedhi/vuetify';

export class FilterOperatingBudgetModalController {
  private filterOperatingBudgetModal!: Modal;
  
  public clean = false;
  
  private checkPage;
  
  public async createfilterOperatingBudgetModal(card) {
    this.filterOperatingBudgetModal = await ModalService.createFromJson('/modals/filterOperatingBudgetModal', true);
    await this.filterOperatingBudgetModal.show();
    const idaprovacaogop = Metadata.getInstance<Select>('IDAPROVACAOGOP');
    this.checkPage = card;
    if (this.checkPage === 'operationalBudgetApproval') {
      idaprovacaogop.isVisible = true;
    } else {
      idaprovacaogop.isVisible = false;
    }
  }
  
  public destroyfilterOperatingBudgetModal() {
    this.filterOperatingBudgetModal?.destroy();
  }
  
  public openfilterOperatingBudgetModal() {
    this.filterOperatingBudgetModal.show();
  }
  
  public closefilterOperatingBudgetModal() {
    this.filterOperatingBudgetModal.hide();
    this.clearInputsFilter();
  }
  
  /**
  * Clears the filter screen fields.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public clearInputsFilter() {
    const form = Metadata.getInstance<Form>('formFilterOperatingBudget');
    const idPeriod = form.value.IDPERIODOORCA;
    form.clearForm();
    form.value.IDPERIODOORCA = idPeriod;
  }
  
  /**
  * Apply the filter made, trigger the actions that define the budget title and organize the visible months.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public async applyFilter() {
    const period = BudgetParameters.getParameters.IDPERIODOORCA;
    const filter = Metadata.getInstance<Form>('formFilterOperatingBudget');
    
    if ((filter.value.NRANOORCAGOP && filter.value.NRPERIODOORCA) || (filter.value.NRANOORCAGOP && period === 'ANUAL')) {
      if (this.checkPage === 'operationalBudgetApproval') {
        const operationalBudgetApprovalModal = Loader.getInstance('OperationalBudgetApprovalController');
        operationalBudgetApprovalModal.filter = filter.value;
        operationalBudgetApprovalModal.filterData = BudgetParameters.treatParameters(filter.value);
        operationalBudgetApprovalModal.initializeFilter();
      } else {
        const operatingBudgetModal = Loader.getInstance('CalculationOperatingBudgetController');
        BudgetParameters.definesMonths(filter.value.NRPERIODOORCA ? filter.value.NRPERIODOORCA.NRPERIODOORCA : 1);
        operatingBudgetModal.filter = filter.value;
        operatingBudgetModal.filterData = BudgetParameters.treatParameters(filter.value);
        if (!this.clean) {
          this.clean = true;
        } else {
          this.clearData();
        }
        operatingBudgetModal.defineTitleBudget();
      }
      this.filterOperatingBudgetModal.hide();
    } else {
      filter.validate();
    }
  }
  
  /**
  * Triggers the function that defines the pages as not loaded, and leaves the first visible page active.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public clearData() {
    const tabsBudget = Metadata.getInstance<ZdTabs>('tabsBudget');
    BudgetParameters.setNotRecharged();
    tabsBudget.activeTab = tabsBudget.tabs.findIndex((obj) => (obj.isVisible === true));
  }
  
  /**
  * Updates the search for units in the filter according to the fields entered.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public createDataSourceFilter() {
    const filial = Metadata.getInstance<SelectMultiple>('CDFILIAL');
    const grupFili = Metadata.getInstance<SelectMultiple>('CDGRUPFILI');
    const areaGere = Metadata.getInstance<SelectMultiple>('CDAREAGERE');
    const segMerc = Metadata.getInstance<SelectMultiple>('CDSEGMER');
    filial.datasource.setFilter({
      CDGRUPFILI: grupFili.value && grupFili.value.length ? grupFili.value.join('_') : 'T',
      CDAREAGERE: areaGere.value && areaGere.value.length ? areaGere.value.join('_') : 'T',
      CDSEGMER: segMerc.value && segMerc.value.length ? segMerc.value.join('_') : 'T',
      
    });
  }
}
