import {
  Modal,
  ModalService,
  SelectMultiple,
  LoadingService,
  Form,
} from '@zeedhi/common';
import {
  Metadata,
  Http,
  I18n,
  Loader,
} from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';

export class ImportDROModalController {
  private importDROModal!: Modal;
  
  public destroyImportDROModal() {
    this.importDROModal?.destroy();
  }
  
  public async openImportDROModal() {
    this.importDROModal = await ModalService.createFromJson('/modals/importDROModal', true);
    await this.importDROModal.show();
  }
  
  public closeImportDROModal() {
    this.importDROModal.hide();
  }
  
  public getFilioperPadr() {
    const destinySelect = Metadata.getInstance<SelectMultiple>('CDFILDEST');
    Http.get('/getFilioperPadr').then((response) => {
      const data = response.data.dataset.filioperPadr;
      destinySelect.datasource.updateData(data);
    });
  }
  
  public confirmImportDRO() {
    const importForm = Metadata.getInstance<Form>('importDROForm');
    if (importForm.validate()) {
      const utilities = new Utilities();
      const dataForm = {
        CDFILDEST: importForm.value.CDFILDEST,
        NRITEMOPERACIONAL: importForm.value.NRITEMOPERACIONAL,
        TITLE: '',
        MESSAGE: I18n.translate('gop::CONFIRM_IMPORT'),
      };
      utilities.showDialog(this.importDRO, dataForm, 'YN');
    }
  }
  
  public importDRO(dataForm, type) {
    const mainController = Loader.getInstance('PercentageAndStructureForBudgetController');
    const utilities = new Utilities();
    dataForm.TYPE = type;
    LoadingService.show();
    Http.post('/replicateImportParams', {
      requestType: 'Row',
      row: dataForm,
    }).then(() => {
      utilities.successMessage();
      mainController.unitRowClick();
    }).catch((e) => {
      utilities.errorMessage();
      console.warn(e);
    }).finally(() => {
      utilities.hideDialogue();
      LoadingService.hide();
    });
  }
}
